@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 #
-# $Id: Logwatch.pm,v 1.12 2005/05/21 22:53:31 bjorn Exp $
+# $Id: Logwatch.pm 150 2013-06-18 22:19:38Z mtremaine $
 
 package Logwatch;
 
@@ -200,7 +200,7 @@ sub TotalCountOrder(\%;&) {
             }
         }
         my $count = $BB <=> $AA;
-        
+
         return $count if $count;
         if (ref $coderef) {
             $a = $A;
@@ -226,10 +226,11 @@ sub canonical_ipv6_address {
     my @a = split /:/, shift;
     my @b = qw(0 0 0 0 0 0 0 0);
     my $i = 0;
-    while (defined $a[0] and $a[0] ne '') {$b[$i++] = shift @a;}
+    # comparison is numeric, so we use hex function
+    while (defined $a[0] and $a[0] ne '') {$b[$i++] = hex(shift @a);}
     @a = reverse @a;
     $i = 7;
-    while (defined $a[0] and $a[0] ne '') {$b[$i--] = shift @a;}
+    while (defined $a[0] and $a[0] ne '') {$b[$i--] = hex(shift @a);}
     @b;
 }
 
@@ -306,16 +307,30 @@ sub LookupIP {
        }
    }
 
-   return $Addr unless($DoLookup);
+   # "Socket" is used solely to get the AF_INET() and AF_INET6()
+   # constants, usually 2 and 10, respectively.  Using Socket is
+   # preferred because of portability, and should be in the standard
+   # Perl distribution.
+   eval "use Socket"; my $hasSocket = $@? 0 : 1;
+   return $Addr unless($DoLookup && $hasSocket);
 
    return $LookupCache{$Addr} if exists ($LookupCache{$Addr});
 
-   if ($Addr =~ /:/ and $Addr !~ /^::ffff:(\d+\.\d+\.\d+\.\d+)/) {
-       return "unresolved IPv6 addr: $Addr";
+   $Addr =~ s/^::ffff://;
+   my $PackedAddr;
+   my $name = "";
+
+   # there are other module functions that do this more gracefully
+   # (such as inet_pton), but we can't guarantee that they are available
+   # in every system, so we use the built-in gethostbyaddr.
+   if ($Addr =~ /^[\d\.]*$/) {
+      $PackedAddr = pack('C4', split /\./,$Addr);
+      $name = gethostbyaddr($PackedAddr,AF_INET());
+   } elsif ($Addr =~ /^[0-9a-zA-Z:]*/) {
+      $PackedAddr = pack('n8', canonical_ipv6_address($Addr));
+      $name = gethostbyaddr($PackedAddr, AF_INET6());
    }
-   $Addr =~ s/::ffff://;
-   my $PackedAddr = pack('C4', split /\./,$Addr);
-   if (my $name = gethostbyaddr ($PackedAddr,2)) {
+   if ($name) {
        my $val = "$Addr ($name)";
        $LookupCache{$Addr} = $val;
        return $val;
@@ -332,7 +347,7 @@ sub LookupIP {
 This function merely prints out some information about --range to STDERR.
 
 =cut
-    
+
 sub RangeHelpDM {
    eval "use Date::Manip"; my $hasDM = $@ ? 0 : 1;
 
@@ -343,6 +358,7 @@ sub RangeHelpDM {
        print STDERR "\nThis system does not have Date::Manip module loaded, and therefore\n";
        print STDERR "the only valid --range parameters are 'yesterday', 'today', or 'all'.\n";
        print STDERR "The Date::Manip module can be installed by using either of:\n";
+       print STDERR "   apt-get install libdate-manip-perl (recommended on Debian)'\n";
        print STDERR "   cpan -i 'Date::Manip'\n";
        print STDERR "   perl -MCPAN -e 'install Date::Manip'\n";
        print STDERR "\nFollowing is a description of the full capabilities available if\n";
@@ -384,7 +400,7 @@ Examples:
    --range '4 hours ago for that hour'
    --range '-3 days'
    --range 'since 2 hours ago for those hours'
-   --range 'between -10 and -2 days'
+   --range 'between -10 days and -2 days'
    --range 'Apr 15, 2005'
    --range 'first Monday in May'
    --range 'between 4/23/2005 and 4/30/2005'
@@ -411,7 +427,7 @@ This function returns the period, which is the part after the "for (those|that|t
 in a range
 
 =cut
-    
+
 sub GetPeriod {
 
    my $range = lc $ENV{"LOGWATCH_DATE_RANGE"} || "yesterday";
@@ -439,6 +455,13 @@ sub TimeBuild {
    my $time = time;
    eval "use Date::Manip"; my $hasDM = $@ ? 0 : 1;
 
+   if ($hasDM) {
+      eval 'Date_TimeZone();';
+      if ($@) {
+         die "ERROR: Date::Manip unable to determine TimeZone.\n\nExecute the following command in a shell prompt:\n\tperldoc Date::Manip\nThe section titled TIMEZONES describes valid TimeZones\nand where they can be defined.\n";
+      }
+   }
+
    my $range = lc $ENV{"LOGWATCH_DATE_RANGE"} || "yesterday";
    my $period = GetPeriod;
    $range =~ s/for\s+(?:those|that|this)\s+((year|month|day|hour|minute|second)s?)\s*$//;
@@ -511,13 +534,11 @@ sub TimeBuild {
 This function returns a regexp to filter by date/time
 
 =cut
-    
+
 
 sub TimeFilter {
    my ($format) = $_[0];
 
-   eval "use Date::Manip"; my $hasDM = $@ ? 0 : 1;
-    
    my $SearchDate;
 
    my $range = lc $ENV{"LOGWATCH_DATE_RANGE"} || "yesterday";
@@ -542,7 +563,7 @@ sub TimeFilter {
            $format =~ s/%a/.../;
            $format =~ s/%d/../;
            $format =~ s/%e/../;
-           if ($period eq 'month') {last;}      
+           if ($period eq 'month') {last;}
            $format =~ s/%b/.../;
            $format =~ s/%m/../;
            if ($period eq 'year') {last;}
@@ -551,7 +572,7 @@ sub TimeFilter {
    }
 
    $SearchDate .= "(";
-    
+
    for my $time (@time_t) {
         if ($time) {
            $SearchDate .= strftime($format, localtime($time)) . "|";
